%wavanalysis
% Function which analyses a .wav file, creating time varying spectral
% outputs which can used for visualization purposes.
%
% [ t_orig, s_orig, Fs, S, F, T, t, s, f, duration_s ] = ...
%    wavanalysis( wavfilename, frame_rate, smooth_amount, STFFT_duration )
%
% wavfilename     Input wave (.wav) filename
% frame_rate       Frame rate /Hz of smoothed output. e.g. 24 frames per
%                  second should be fine for video purposes in real time.
% smooth_amount    Smoothing parameter 0....1
% STFFT_duration   Duration in seconds on Short Time Fast Fourier Transform
%                  used to determine the output spectrum. If this is equal or longer than
%                  the duration of the recording a single vector will be produced.
% plots            Create spectrograms, time series etc
% f_max_Hz         Maximum frequency for plotting /Hz
%
% t_orig           Time samples /s or original recording
% s_orig           Amplitude samples of original recording
% Fs               Sample rate /Hz of .wav file
% S                Short time FFT logarithmic amplitude. Rows are frequency,
%                  columns are time samples
% F                Matrix of frequency values /Hz corresponding to S
% T                Matrix of time sample values /s corresponding to S
% t                Time sample values corresponding to smoothed output at frame
%                  rate frame_rate /Hz
% s                (Signed) smoothed signal amplitude at frame rate frame_rate /Hz
% f                Smoothed dominant frequency /Hz at time samples t
% duration_s       Duration /s of recording

function [ t_orig, s_orig, Fs, S, F, T, t, s, f, duration_s ] = ...
    wavanalysis( wavfilename, frame_rate, smooth_amount, STFFT_duration, plots, f_max_Hz  )

%Load wavfile. s_orig is amplitude vector, Fs is sample frequency /Hz
%Amplitude vector is always scaled to be in the range [-1,1]
if exist(wavfilename,'file')
    [s_orig,Fs,bits] = wavread(wavfilename);
else
    %Default file is an excerpt from Handel's Halleluja Chorus
    [s_orig,Fs,bits] = wavread('handel.wav');
end

%Combine left and right stereo channels
s_orig = sum(s_orig,2);

%Convert to a row vector
s_orig = s_orig.';

%Duration of recording /s
duration_s = length(s_orig)/Fs;

%

% COMPUTE SHORT TIME FOURIER TRANSFORM %

%Determine number of samples which corresponds to STFFT
STFFT_duration = min( STFFT_duration, duration_s );
num_in_STFFT = round( Fs * STFFT_duration );

%Determine number of STFFTs
num_STFFTs = ceil( length(s_orig) / num_in_STFFT );

%Pad signal with zeros to make sure it has total number of samples =
%num_s_in_STFFT x num_STFFTs
if length(s_orig) < num_STFFTs * num_in_STFFT
    dim = size(s_orig);
    s_orig = [ s_orig, zeros(1, num_STFFTs * num_in_STFFT - length(s_orig) ) ];
end

%Determine original time sample array
t_orig = linspace(0, duration_s, length(s_orig) );

%Transform interpolated y vector into a matrix of N rows * 2*N columns
S = reshape(s_orig,[num_in_STFFT,num_STFFTs]);

%Compute weighted STFT over rows for each column.
w = dolphcheb(num_in_STFFT,40);
w = repmat(w.',[1,num_STFFTs]);
S = fft( w.*S ,[],1 );
S = 10*log10(abs(S));
S( S<0 ) = 0;
S = S/max(max(S));

%Ignore -ve frequencies. Since signal only contains amplitude information
%it will be symmetric. Assume no frequencies are dominant which are over
%half the sample rate!
S = S(1:round(num_in_STFFT/2),:);

%Define fractional time and frequencies for spectrogram
F = linspace(0, Fs/2, round(num_in_STFFT/2) );
T = linspace(0,duration_s,num_STFFTs);
[F,T]=meshgrid(F,T);

%

% SMOOTH SIGNAL AND FIND DOMINANT FREQUENCIES

%Compute dominant frequency /Hz for each time interval
f = zeros(1,num_STFFTs);
FF = linspace(0, Fs/2, round(num_in_STFFT/2) );
fd=[];
for n=1:num_STFFTs
    fd = FF(  S(:,n) == max( S(:,n) ) ) ;
    f(n) = fd(1);
end
clear fd
clear FF

%Interpolate at desired frame rate
t = linspace( 0, duration_s, round( frame_rate * duration_s ) );
s = interp1( linspace( 0, duration_s, length(s_orig)), s_orig, t );
f = interp1( linspace( 0, duration_s, length(f) ), f, t );

%Apply smoothing 
f = smooth(f,smooth_amount);
s = smooth(s,smooth_amount);

%Create plots?
if plots==1
    if isempty(f_max_Hz)
        f_max_Hz = max(f);
    end
    wavanalysis_plot( f_max_Hz, t_orig, s_orig, Fs, S, F, T, t, s, f,...
    duration_s,wavfilename, frame_rate, smooth_amount, STFFT_duration );
end

%%

% Wav analysis plotter
function wavanalysis_plot( f_max_Hz, t_orig, s_orig, Fs, S, F, T, t, s, f,...
    duration_s,wavfilename, frame_rate, smooth_amount, STFFT_duration  )

%Plot dominant frequency and signal against time
figure('color',[1 1 1]);

%Frequency
subplot(2,1,1);
plot( t, f );
title( ['Dominant frequency vs time for ',wavfilename ,...
    '. Frame rate = ',num2str(frame_rate),'Hz. STFFT duration = ',num2str(STFFT_duration),'s'] );
xlabel('Time /s')
ylabel('Dominant frequency /Hz')
axis tight
ylim([0,f_max_Hz]);

%Signal amplitude
subplot(2,1,2);
plot( t, s );
title( ['Signal amplitude vs time for ',wavfilename ,...
    '. Frame rate = ',num2str(frame_rate),'Hz. STFFT duration = ',num2str(STFFT_duration),'s'] );
xlabel('Time /s')
ylabel('Signal amplitude')
axis tight

%Create PNG image
print( gcf, 'wavanalysis freq amp t plot','-dpng','-r300' );
close(gcf);

%

%Plot spectrogram
figure('color',[1 1 1]);
pcolor(T,F,S.');
view(2);
shading interp
axis tight
ylim([0,f_max_Hz]);
colorbar
title( ['Spectrogram for ',wavfilename ,'. Frame rate = ',...
    num2str(frame_rate),'Hz. STFFT duration = ',num2str(STFFT_duration),'s'] )
xlabel('Time /s')
ylabel('Frequency /Hz')
print( gcf, 'wavanalysis spectrogram','-dpng','-r300' );
close(gcf);

%%

% Demonstration of wavanalysis
function wavanalysis_demo

%%%% INPUTS PARAMETERS %%%%

wavfilename = 'handel.wav';
frame_rate = 24;
smooth_amount = 0.5;
STFFT_duration = 0.2;
plots = 1;

%Highest frequency /Hz (Note if this higher than the sample rate, half the
%sample rate will be taken as the maximum)
f_max_Hz = 2000;

%Perform wave file analysis
[ t_orig, s_orig, Fs, S, F, T, t, s, f, duration_s ] = ...
    wavanalysis( wavfilename, frame_rate, smooth_amount, STFFT_duration, plots, f_max_Hz );

%%

%DOLPHCHEB
% Function that computes Dolph-Chebyshev windowing function. The main role
% of the window (as with all windows) is to damp out the effects of the
% Gibbs phenomenon that results from truncation of an infinite series (e.g.
% a Discrete Fourier Transform). Note this window is complex
% and can therefore only really be visualised when the amplitude of the 
% window DFT is plotted (in dB).
%
% LAST UPDATED by Andrew French, AMS Ltd. 27/08/2004.
%
% Syntax: w=dolphcheb(N,sl_level)
%
% N      - Length of window sl_level. This ifference between peak and
%          sidelobes /dB in the DFT of the window.
% w      - Window 'weights'. These are real, positive numbers.

function w=dolphcheb(N,sl_level)
gamma=sl_level/20;
if N>1
    M=N-1;
    alpha=cosh(acosh(10^gamma)/M);
    for n=1:M
        A(n)=((-1)^(n-1))*cos(M*acos(alpha*cos(pi*(n-1)/M)));
    end
    B=fft(A);
    B(1)=0.5*B(1);
    B(N)=B(1); 
    w=B/max(B);
else
    w=1;    
end

%%

%smooth
% Smoothing function. 
% x is the input vector of values to smooth
% K is the smoothing constant 0 < K < 1. K=1 implies total smoothing, K=0
% implies no smoothing at all.
function y = smooth( x, K )
y=zeros(size(x));
Y=x(1);
for n=1:length(x)
    y(n) = (1-K) * x(n) + K * Y;
    Y = y(n);
end

%End of code





